#!/usr/bin/python2

import getopt, sys
from math import sqrt

def usage():
    print("Parses ARL NOAA text files and outputs table with following columns:")
    print("Height(m)   Temp(deg)     dT/dH   Wind(m/s)    Dir(deg)     Length,(m)")
    print("Wavelength is calculated only for stable conditions")
    
    print("Options:")
    print("     -i, --input FILE    - use ARL NOAA text file")
    print("     -t, --time TIME     - output data for given time (now its a column number)")
    print("     -o, --output FILE   - intput file (stdout by default)")
    exit(0)
    
def parseData( inputFile, startTag, dictTag, heightList, columnIndex):
    columnsNumber = 0
    switchIndex = 0
        
    inputFile.seek(0)
    
    for s in inputFile.readlines():
        if switchIndex>=3:
            index = switchIndex-3
            columns = s.split()
            if len(columns) < columnsNumber:
                break
            
            columnsNumber = len(columns)
            height = float( columns[0] )
            
        
            if index >= len(heightList):
                heightList.append( dict() )
                
            heightList[ index ][ dictTag ] = float( columns[ columnIndex ] )
            switchIndex += 1
            
        elif switchIndex == 0:
            if s.find( startTag ) >= 0:
                switchIndex = 1
        else:
            switchIndex += 1
            
            
        
    
    
def parseNoaaFile( inputFile, outputFile, timeColumn ):
    
    startTags = [   ("========Temperature Deg C=====","Temp(deg)"),\
                    ("========Wind Direction========","Dir(deg)"),\
                    ("========Wind Speed m/s=========","Wind(m/s)") ]
    
    heightList = list()
    
    parseData( inputFile, "========Temperature Deg C=====", "Height(m)", heightList, 0 )
    
    for s,t in startTags:
        parseData( inputFile, s, t, heightList, int(timeColumn) )
        
    for i in xrange(1, len(heightList)-1 ):
        prevH = heightList[i-1]["Height(m)"]
        prevT = heightList[i-1]["Temp(deg)"]
        H = heightList[i]["Height(m)"]
        T = heightList[i]["Temp(deg)"]
        wind = heightList[i]["Wind(m/s)"]
        
        dH = H - prevH
        dT = T - prevT
        
        strat = 100*dT/dH
        
        heightList[i]["dT/dH"] = strat
        
        try:
            wavelen = 344*wind/sqrt( 1 + strat )
            heightList[i]["Length(m)"] = wavelen
        except:
            pass
            
        
    printTags=["Height(m)","Temp(deg)","dT/dH","Wind(m/s)","Dir(deg)","Length(m)"]
    
    #print( heightList )
    
    for tag in printTags:
        outputFile.write( ( "% 12s" % tag ) )
    outputFile.write("\n")
        
        
    for c in heightList:
        for tag in printTags:
            
            value = float(0)
            try:
                value = c[tag]
            except:
                pass
                
            outputFile.write( ( "% 12.2f" % value ) )
            
        outputFile.write("\n")
           

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hi:o:t:", ["help", "input=", "output=","time="])
    except getopt.GetoptError as err:
        # print help information and exit:
        print( str(err) ) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)
    outputFile = sys.stdout
    inputFile = None
    timeColumn = 1
    verbose = False
    for o, a in opts:
        if o == "-v":
            verbose = True
        elif o in ("-h", "--help"):
            usage()
            sys.exit()
        elif o in ("-o", "--output"):
            outputFile = a
        elif o in ("-i", "--input"):
            inputFile = open( a, "r" )
        elif o in ("-t", "--time"):
            timeColumn = a
        else:
            assert False, "unhandled option"
            
    if not inputFile:
        print("Incorrect usage: select input file!")
        exit(1)
            
    parseNoaaFile( inputFile, outputFile, timeColumn )
    
    

if __name__ == "__main__":
    main()
